<?php
/**
 * 2007-2021 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    PrestaShop SA <contact@prestashop.com>
 *  @copyright 2007-2021 PrestaShop SA
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */
class TarkpayValidationModuleFrontController extends ModuleFrontController
{
    /**
     * This class should be use by your Instant Payment
     * Notification system to validate the order remotely
     */
    public function postProcess()
    {
        $choosedSwift = null;
        if (isset($_POST['choosedSwift'])){
            $choosedSwift = $_POST['choosedSwift'];
        }
        $cart = $this->context->cart;
		if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active)
			Tools::redirect('index.php?controller=order&step=1');

		// Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
		$authorized = false;
		foreach (Module::getPaymentModules() as $module)
			if ($module['name'] == 'tarkpay')
			{
				$authorized = true;
				break;
			}
		if (!$authorized)
			die($this->module->getTranslator()->l('This payment method is not available.'));

		$customer = new Customer($cart->id_customer);
		if (!Validate::isLoadedObject($customer))
			Tools::redirect('index.php?controller=order&step=1');
		$currency = $this->context->currency;
		$total = (float)$cart->getOrderTotal(true, Cart::BOTH);
		$returnUrl = Tools::getHttpHost(true).__PS_BASE_URI__ .'index.php?controller=order-confirmation&id_cart='.$cart->id.'&id_module='.$this->module->id.'&id_order='.$this->module->currentOrder.'&key='.$customer->secure_key;
        $apiKey = $this->module->api_key;
        $requestUrl = $this->module->url . '/payment';
        if ($this->isValidOrder() === true) {
            $payment_status = Configuration::get('TARKPAY_OS_WAITING');
            $message = null;
        } else {
            $payment_status = Configuration::get('PS_OS_ERROR');
            $message = $this->module->l('An error occurred while processing payment');
        }
        $this->module->validateOrder($cart->id, $payment_status, $total, $this->module->displayName, $message, array(), (int)$currency->id, false, $customer->secure_key);
        $order = new Order($cart->id);
        $reference = $order->reference;
        $payment = $this->initiateTarkpayPayment($requestUrl, $total, $cart->id, $returnUrl, $apiKey, $reference, $choosedSwift);
        if(isset($payment['success']) && $payment['success'] == 1){
            if(isset($payment['id'])){
                $order->addOrderPayment($total, $this->module->displayName, $payment['id']);
                //$orderPaymentDatas = OrderPayment::getByOrderId($orderId);
            }
            if(isset($payment['_links'][$choosedSwift])){
                return Tools::redirect($payment['_links'][$choosedSwift]);
            }
            return Tools::redirect($payment['_links']['page']);
        } else {
            return Tools::redirect('/error'); #TODO ::
        }
    }
    protected function isValidOrder(){
        return true; #TODO ::
    }
    protected function initiateTarkpayPayment($requestUrl, $total, $cartId, $returnUrl, $apiKey, $reference, $choosedSwift)
    {
        $postData =
            array(
                'amount' => $total,
                'description' => 'Invoice nr. '. $cartId. ', '. $reference,
                'redirect_url' => $returnUrl,
                'partner_payment_reference' => $cartId
            );

        $ch = curl_init($requestUrl);
        curl_setopt_array($ch, array(
            CURLOPT_POST => TRUE,
            CURLOPT_RETURNTRANSFER => TRUE,
            CURLOPT_HTTPHEADER => array(
                'Authorization: Bearer '. $apiKey,
                'Content-Type: application/json'
            ),
            CURLOPT_POSTFIELDS => json_encode($postData)
        ));

        $response = curl_exec($ch);
        if($response === FALSE){
            die(curl_error($ch));
        }

        $responseData = json_decode($response, TRUE);
        curl_close($ch);
        return $responseData;
    }
}
